/*
 Author: Tonetta Ivan (Ruben) - 2016.03 - Italy - Cabbage 1.02
- many thanks to God [Yahwéh, YHWH, Geova]: He made life possible, and all you can do with it;
- many thanks to my wife and my children for their patience;
- many thanks to Rory Walsh for Cabbage and his great support (http://forum.cabbageaudio.com/);
- many thanks to Iain McCurdy, for his support and all code shared (http://iainmccurdy.org/);
- many thanks to Riccardo Bianchini, Alessandro Cipriani, Giorgio Zucco, Jim Aikin for their wonderful books and "Vallste" for his (old) support (http://triceratupuz.altervista.org/";
- many thanks to Barry Vercoe and all Csound developers in the world (http://www.csounds.com/);

I'm not a writer of professional code in csound; so what I present is a very simple Lissajous functions generator; it uses a set of 16 tables to generate differents graphical and timbrical results.
Very interesting is when is used tables with 16 or 2 point for generating sine wave with oscil (note: without interpolation). Analyzing the audio generated by an analog oscilloscope, 
you can see some really strange curves; the preset "Nervous_point" It is my favorite... Ideal for mixing with VJ effects softwares like "Resolume Arena"...

*/
<Cabbage>
form size(510, 370), caption("2016.03.10_Cabbage_For_Lissajous"), pluginID("plu1")
keyboard   bounds(000,277,510,050), middlec(1)
#define SLIDER_PARAMS  textbox(1), colour(000,000,255,050), trackercolour(000,000,255,255)
groupbox   bounds(0, 4,250, 170), colour(160, 160, 160, 20), text("Vco1"), plant("VCO"){ 
rslider    bounds(000,025,100,100), channel("note1"),    range(-96,   096,  0,     1,  1    ), text("Note"),  $SLIDER_PARAMS
rslider    bounds(070,025,100,100), channel("fine1"),    range(-1,    001,  .05,   1,  .001 ), text("Fine"),  $SLIDER_PARAMS 
rslider    bounds(140,025,100,100), channel("amp1"),     range(.0001, 1,    .5,    .5, .0001), text("Amp"),  $SLIDER_PARAMS
combobox   bounds(070,140,100,020), channel("combobox1"), items("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "Square", "Saw-Up", "Saw-Dwn", "Tri"), colour("blue"), fontcolour("black")
}
groupbox   bounds(256, 8, 250, 170), colour(160, 160, 160, 20), text("Vco2"), plant("VCO"){ 
rslider    bounds(000,025,100,100), channel("note2"),    range(-96,   096,  0,     1,  1    ), text("Note"),  $SLIDER_PARAMS
rslider    bounds(070,025,100,100), channel("fine2"),    range(-1,    001,  0,     1,  .001 ), text("Fine"),  $SLIDER_PARAMS
rslider    bounds(140,025,100,100), channel("amp2"),     range(.0001, 1,    .5,    .5, .0001), text("Amp"),   $SLIDER_PARAMS
combobox   bounds(070,140,100,020), channel("combobox2"), items("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "Square", "Saw-Up", "Saw-Dwn", "Tri"), colour("blue"), fontcolour("black")
}
groupbox   bounds(0, 176, 250, 100),                                                                                 colour(160,160,160,020), text("ADSR"), plant("ADSR"){ 
rslider    bounds(0, 25, 70, 70), channel("a1"),     range(.001,005,0.001,      .5, 0.01),   text("A"),     $SLIDER_PARAMS
rslider    bounds(40, 25, 70, 70), channel("d1"),     range(.001,002,0.200,      .5, 0.01),   text("D"),     $SLIDER_PARAMS
rslider    bounds(80, 25, 70, 70), channel("s1"),     range(.001,001,0.500,      1,  0.01),   text("S"),     $SLIDER_PARAMS
rslider    bounds(120, 25, 70, 70), channel("r1"),     range(.001,002,0.100,      .5, 0.01),   text("R"),     $SLIDER_PARAMS
rslider    bounds(180, 25, 70, 70), channel("master"), range(.001,0.8,0.500,      1,  0.01),   text("Master"), $SLIDER_PARAMS
}
groupbox   bounds(000,335,380,027), linethickness(0), colour(160,160,160,020), plant("Background"){ 
combobox   bounds(170,004,160,020), channel("combobox"), populate("*.snaps")
filebutton bounds(335,004,040,020), channel("save"), text("Save"), populate("*.snaps")
label      bounds(004,006,180,015), text("Tonetta Ivan - 2016.03"), align("left"), fontcolour("blue")
}
rslider bounds(248, 176, 97, 93), channel("arprate"), range(0, 8, 0, .5, .25),text("Arp Rate") $SLIDER_PARAMS

checkbox bounds(432, 208, 75, 20), channel("record"), text("Record"), colour("red"), identchannel("record_ident")
button   bounds(432, 184, 75, 20), colour("red"), text("New File","New File"), channel("reset"), value(1), latched(0)
checkbox bounds(432, 232, 75, 20), channel("play"), text("Play"), colour("green"), 
rslider bounds(312, 176, 108, 93), channel("arptable"), range(1, 3, 1, 1, 1),text("Arp Table") $SLIDER_PARAMS
checkbox bounds(432, 256, 80, 20), channel("arptype"), text("Arp Type")
</Cabbage>
<CsoundSynthesizer>
<CsOptions>
-n -d -+rtmidi=NULL -M0 -m0d ;--midi-velocity-amp=5 ; --midi-key-cps=4
</CsOptions>
<CsInstruments>
sr = 44100
ksmps = 16
nchnls = 2
0dbfs = 1


instr 1
; Streaming inputs for GUI values 
iatt1      chnget "a1"
idec1      chnget "d1"
isus1      chnget "s1"
irel1      chnget "r1"
knote1     chnget "note1"
kfine1     chnget "fine1"
kamp1      chnget "amp1"
icombobox1 chnget "combobox1"
knote2     chnget "note2"
kfine2     chnget "fine2"
kamp2      chnget "amp2"
icombobox2 chnget "combobox2"
kmaster    chnget "master"
; smooth for all k variables
knote1  portk knote1,  0.01
kfine1  portk kfine1,  0.01
kamp1   portk kamp1,   0.01
knote2  portk knote2,  0.01
kfine2  portk kfine2,  0.01
kamp2   portk kamp2,   0.01
kmaster portk kmaster, 0.01
; MIDI input
imidi notnum 

;add a new note tables here, check the slider value and
;enter to that table. arprate and table only change between notes
;this mean you can use several tables at once across different notes
if chnget:i("arprate")>0 then
	if chnget:i("arptable")==1 then
		kArp oscil 1/4, chnget:i("arprate"), 100
	elseif chnget:i("arptable")==2 then
		kArp oscil 1, chnget:i("arprate"), 101
		kArp = kArp/3
	elseif chnget:i("arptable")==3 then
		kArp oscil 1, chnget:i("arprate"), 102
		kArp = kArp/2
	endif
else
	kArp = 1
endif

; convert in cps the result of summing for all the midi number notes available
knote1 cpsmidinn  imidi + knote1 + kfine1
knote2 cpsmidinn  imidi + knote2 + kfine2
knote1 = knote1*kArp
knote2 = knote2*kArp
; amp global envelope
kenv madsr iatt1, idec1, isus1, irel1, 0, 1
; Oscil for L channel
if    (icombobox1 == 1) then
	aL oscil kamp1 * 1.5, knote1*kArp, 1
elseif(icombobox1 == 2) then
	aL oscil kamp1,       knote1, 2
elseif(icombobox1 == 3) then
	aL oscil kamp1,       knote1, 3
elseif(icombobox1 == 4) then
	aL oscil kamp1,       knote1, 4
elseif(icombobox1 == 5) then
	aL oscil kamp1,       knote1, 5
elseif(icombobox1 == 6) then
	aL oscil kamp1,       knote1, 6
elseif(icombobox1 == 7) then
	aL oscil kamp1,       knote1, 7
elseif(icombobox1 == 8) then
	aL oscil kamp1,       knote1, 8
elseif(icombobox1 == 9)  then
	aL oscil kamp1,       knote1, 9
elseif(icombobox1 == 10) then
	aL oscil kamp1,       knote1, 10
elseif(icombobox1 == 11) then
	aL oscil kamp1,       knote1, 11
elseif(icombobox1 == 12) then
	aL oscil kamp1,       knote1, 12
elseif(icombobox1 == 13) then
	aL oscil kamp1,       knote1, 13
elseif(icombobox1 == 14)  then
	aL oscil kamp1,       knote1, 14
elseif(icombobox1 == 15) then
	aL oscil kamp1,       knote1, 15
elseif(icombobox1 == 16) then
	aL oscil kamp1,       knote1, 16
endif

; Oscil for R channel
if    (icombobox2 == 1)  then
	aR oscil kamp2 * 1.5, knote2, 1
elseif(icombobox2 == 2)  then
	aR oscil kamp2,       knote2, 2
elseif(icombobox2 == 3)  then
	aR oscil kamp2,       knote2, 3
elseif(icombobox2 == 4)  then
	aR oscil kamp2,       knote2, 4
elseif(icombobox2 == 5)  then
	aR oscil kamp2,       knote2, 5
elseif(icombobox2 == 6)  then
	aR oscil kamp2,       knote2, 6
elseif(icombobox2 == 7)  then
	aR oscil kamp2,       knote2, 7
elseif(icombobox2 == 8)  then
	aR oscil kamp2,       knote2, 8
elseif(icombobox2 == 9)  then
	aR oscil kamp2,       knote2, 9
elseif(icombobox2 == 10) then
	aR oscil kamp2,       knote2, 10
elseif(icombobox2 == 11) then
	aR oscil kamp2,       knote2, 11
elseif(icombobox2 == 12) then
	aR oscil kamp2,       knote2, 12
elseif(icombobox2 == 13)  then
	aR oscil kamp2,       knote2, 13
elseif(icombobox2 == 14)  then
	aR oscil kamp2,       knote2, 14
elseif(icombobox2 == 15) then
	aR oscil kamp2,       knote2, 15
elseif(icombobox2 == 16) then
	aR oscil kamp2,       knote2, 16
endif

amixL =  (aL * kenv) * kmaster
amixR = (aR * kenv) * kmaster

aL dcblock amixL
aR dcblock amixR

outs aL, aR

gaL = (aL * kenv) * kmaster 
gaR = (aR * kenv) * kmaster
endin


/* Iain McCurdy */
gkRecordingActiveFlag	init	0
gkFileRecorded		init	0


instr	100

 gkrecord	chnget	"record"
 gkreset	chnget	"reset"
 gkplay		chnget	"play"
 kRecStart	trigger	gkrecord,0.5,0
 if kRecStart==1&&gkRecordingActiveFlag==0 then
  event	"i",9000,0,-1
  gkRecordingActiveFlag	=	1
  chnset "text(\"Record\")", "record_ident" 
 endif
 kPlayStart	trigger	gkplay,0.5,0
 if kPlayStart==1&&gkFileRecorded==1 then
  event	"i",9001,0,3600
 endif

 kResetTrig	trigger	gkreset,0.5,1
 if kResetTrig==1 && gkRecordingActiveFlag==1 then
  event	"i",9000,0,-1
 endif  
  
endin


instr 9000
 gkFileRecorded		init	1
 itim     date
 Stim     dates     itim
 itim     date
 Stim     dates     itim
 Syear    strsub    Stim, 20, 24
 Smonth   strsub    Stim, 4, 7
 Sday     strsub    Stim, 8, 10
 iday     strtod    Sday
 Shor     strsub    Stim, 11, 13
 Smin     strsub    Stim, 14, 16
 Ssec     strsub    Stim, 17, 19
 Sfilnam  sprintf  "%s_%s_%02d_%s_%s_%s.wav", Syear, Smonth, iday, Shor,Smin, Ssec
 gSname   sprintf "FileRecorder_%s", Sfilnam
 
 if changed:k(gkrecord)==1 then
 	if gkrecord == 1 then
 	chnset "text(\"Stop\")", "record_ident"
 	else
 	chnset "text(\"Record\")", "record_ident"
 endif
 endif
 
 if gkrecord==1 then			; record
  fout gSname, 8, gaL, gaR
 endif

endin

instr	9001
 if gkplay==0 then
  turnoff
 endif
  aL,aR	diskin2	gSname,1
  	outs	aL,aR
 iFileLen	filelen	gSname
 p3	=	iFileLen
 xtratim	0.1 
 krelease	release
 printk2	krelease
 	chnset	1-krelease,"play"
endin

</CsInstruments>  
</CsInstruments>  
<CsScore>

f1  0 4096 10 1
f2  0 2048 10 1
f3  0 1024 10 1
f4  0  512 10 1
f5  0  256 10 1
f6  0  128 10 1
f7  0   64 10 1
f8  0   32 10 1
f9  0   16 10 1
f10 0    8 10 1
f11 0    4 10 1
f12 0    2 10 1
f13 0  512  7 1 256 1 0 -1 256 -1 ; Square
f14 0  512  7 -1 512 1 ; Saw-Up
f15 0  512  7 1 512  -1 ; Saw-Down
f16 0  512  7 -1 256 1 256 -1 ; Triangle

;add any new arpeggio table here
;make sure to increase update the arl table slider range
;add add a check for it in instrument 1
f100 0 8 -2 1 2 3 1 3 4 3 1
f101 0 8 -2 1 0 3 2 0 2 0 4 
f102 0 8 -2 1 2 0 2 0 1 2 3
f0 z

i100 0 [60*60*24*7]
</CsScore>
</CsoundSynthesizer>